//+------------------------------------------------------------------+
//|                                               VolumeDeltaWPR.mq5 |
//|                                      Copyright  2016, Marketeer |
//+------------------------------------------------------------------+
#property copyright "Copyright  2016, Marketeer"
#property link      "https://www.mql5.com/en/users/marketeer"
#property version   "1.0"
#property description "Volume Delta WPR\n"
#property description "WPR formula applied on VolumeDelta indicator (should be obtained separately)"

#property strict

#property indicator_separate_window

#property indicator_buffers 1
#property indicator_plots   1

#property indicator_label1  "WPR"
#property indicator_type1   DRAW_LINE
#property indicator_color1  clrRed
#property indicator_style1  STYLE_SOLID
#property indicator_width1  2

extern int WPRPeriod = 12;
extern string Engine = "VolumeDelta";
extern int EngineBuffer = 3 /* meaning depends from engine */;

sinput string EngineInfo = "=====   P A R A M E T E R S   =====";

       int PointsPerCell = 1000;
extern int LastBars = 1000;
extern int CumulativePeriod = 12;
      bool StickToBar0 = true;
       int Corner = 0;
       int CellWidth = 40;
       int CellHeight = 15;
     color ColorBG = clrBlack;
       int FontSize = 0;
      bool HideTable = true;
      bool HideMark = true;
extern bool AdditionalBoolean = true /* depends from specific engine: HideBuySell or UseRealVolumes or HideArrow ... */;
extern int AdditionalValue = 0;

double Buffer[];
double v[];
int handle;

const string title = "WPR (" + (string)WPRPeriod + ") << " + Engine + "[" + (string)EngineBuffer + "]";

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
{
  SetIndexBuffer(0, Buffer);
  ArraySetAsSeries(Buffer, true);
  ArrayResize(v, WPRPeriod);

#ifdef __MQL5__
  IndicatorSetString(INDICATOR_SHORTNAME, title);
  handle = iCustom(Symbol(), Period(), Engine,
    PointsPerCell,
    LastBars,
    CumulativePeriod,
    StickToBar0,
    Corner,
    CellWidth,
    CellHeight,
    ColorBG,
    FontSize,
    HideTable,
    HideMark,
    AdditionalBoolean, //UseRealVolumes
    AdditionalValue
  );
  
  if(handle != INVALID_HANDLE)
  {
    return(INIT_SUCCEEDED);
  }
  Print("Engine ", Engine, " not initialized");
  return(INIT_FAILED);
#else
  IndicatorShortName(title);
  IndicatorBuffers(1);
  return(INIT_SUCCEEDED);
#endif  
}
  
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total, const int prev_calculated, 
                const datetime& time[], const double& open[], const double& high[], const double& low[], const double& close[], 
                const long& tick_volume[], const long& volume[], const int& spread[])
{
  for(int i = rates_total - MathMax(prev_calculated, 1); i >= 0; i--)
  {
    if(i + WPRPeriod < rates_total)
    {
      for(int j = 0; j < WPRPeriod; j++)
      {
#ifdef __MQL5__
        double result[1];
        if(CopyBuffer(handle, EngineBuffer, i + j, 1, result) == 1)
        {
          v[j] = result[0];
        }
        else
        {
          v[j] = EMPTY_VALUE;
        }
#else
        v[j] = iCustom(Symbol(), Period(), Engine,
                PointsPerCell,
                LastBars,
                CumulativePeriod,
                StickToBar0,
                Corner,
                CellWidth,
                CellHeight,
                ColorBG,
                FontSize,
                HideTable,
                HideMark,
                AdditionalBoolean,
                AdditionalValue,
                EngineBuffer,
                i + j
              );
#endif      
      }
      
      Buffer[i] = Calc(v);
      
    }
    else
    {
      Buffer[i] = EMPTY_VALUE;
    }
  }
  return(rates_total);
}
//+------------------------------------------------------------------+

double Calc(double &a[])
{
  int hh = ArrayMaximum(a);
  int ll = ArrayMinimum(a);
  double d = a[hh] - a[ll];
  if(d == 0) return(EMPTY_VALUE);

  double r = (a[0] - a[ll]) / d;
  return(r);
}
